package org.moviepedia;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.moviepedia.moviedb.Movie;
import org.moviepedia.moviedb.MovieSearch;

import android.app.Activity;
import android.database.DataSetObserver;
import android.graphics.drawable.Drawable;
import android.text.format.DateUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ListAdapter;
import android.widget.TextView;

import org.moviepedia.R;

public class SearchResultAdapter implements ListAdapter {

	private static final String TAG = SearchResultAdapter.class.getName();
	
	private final Activity activity_;
	private final MovieSearch search_;
	private final List<Movie> movies_;
	private final Iterator<Movie> iterator_;
	
	public SearchResultAdapter(Activity activity, MovieSearch search) {
		activity_ = activity;
		search_ = search;
		iterator_ = search_.iterator();
		movies_ = new ArrayList<Movie>( search_.getTotalMovies() );
	}

	public int getCount() {
		return search_.getTotalMovies();
	}

	public Object getItem(int position) {
		return position;
	}

	public long getItemId(int position) {
		return position;
	}

	public int getItemViewType(int position) {
		return R.layout.list_movie;
	}

	public View getView(int position, View convertView, ViewGroup parent) {

		LinearLayout layout = null;
        if (convertView == null) {
        	LayoutInflater li = LayoutInflater.from(activity_);
        	layout = (LinearLayout)li.inflate(R.layout.list_movie, null);
        } else {
        	layout = (LinearLayout)convertView;
        }
		
		while ( movies_.size() <= position && iterator_.hasNext() )
		{
			movies_.add( iterator_.next() );
		}
		
		Movie movie = movies_.get(position);
		if ( movie != null )
		{
			TextView title = (TextView)layout.findViewById(R.id.list_movie_title);
			title.setText( movie.getTitle() );

			Date releaseDate = movie.getReleaseDate();
			if ( releaseDate != null )
			{
				TextView year = (TextView)layout.findViewById(R.id.list_movie_year);
				String dateTime = DateUtils.formatDateTime(activity_, releaseDate.getTime(), 0);
				year.setText( dateTime );
			}
			
			InputStream inputStream = movie.getPoster();
			if ( inputStream != null )
			{
				ImageView poster = (ImageView)layout.findViewById(R.id.list_movie_poster);
				Drawable picture = Drawable.createFromStream( inputStream, movie.getTitle() );
				poster.setImageDrawable( picture );
			}
		}
		return layout;
	}

	public int getViewTypeCount() {
		return 1;
	}

	public boolean hasStableIds() {
		return true;
	}

	public boolean isEmpty() {
		return search_.getTotalMovies() == 0;
	}

	public void registerDataSetObserver(DataSetObserver observer) {
		Log.e(TAG, "method not implemented: registerDataSetObserver");
	}

	public void unregisterDataSetObserver(DataSetObserver observer) {
		Log.e(TAG, "method not implemented: unregisterDataSetObserver");
	}

	public boolean areAllItemsEnabled() {
		return true;
	}

	public boolean isEnabled(int position) {
		return true;
	}
}