package org.moviepedia.moviedb;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;

import org.apache.http.HeaderElement;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.protocol.BasicHttpContext;
import org.apache.http.protocol.HTTP;
import org.apache.http.protocol.HttpContext;

import com.google.gson.Gson;

class _MovieDbImpl implements MovieDb {

	_ConfigurationResponse configResponse_ = null;
	Gson gson_;
	
	public _MovieDbImpl() {
		gson_ = new Gson();
	}
	
	public void connect()
	{
		HttpResponse response = _getResponse( _DbConfig.CONFIGURATION_URL );
		String body = _getResponseBody(response);
		processConfigurationResponse(body);
	}

	public MovieSearch search(String query) {
		if ( configResponse_ == null )
		{
			throw new DbConfigException("Configuration not available, the connection might have failed");
		}
		return new _MovieSearchImpl(this, query);
	}

	public void disconnect() {
		configResponse_ = null;
	}

	private void processConfigurationResponse(String body)
	{
		configResponse_ = gson_.fromJson(body, _ConfigurationResponse.class);
	}
	
	HttpResponse _getResponse(String url)
	{
		HttpClient client = new DefaultHttpClient();
		HttpContext context = new BasicHttpContext();
		try {
			HttpGet request = new HttpGet( url );
			
			_setHeaders(request);
			
			HttpResponse response = client.execute(request, context);
			int statusCode = response.getStatusLine().getStatusCode();
			if ( statusCode != _DbConfig.HTTP_OK )
			{
				throw new DbConnectionException("Http error: " + statusCode);
			}
			return response;
		} catch (IOException e) {
			throw new DbConnectionException("Error trying to connect", e);
		}
	}

	// the web site api.themoviedb.org fails randomly if this header is not set 
	private void _setHeaders(HttpGet request) {
		request.setHeader("Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8");
	}
	
	InputStream _getInputStream(HttpResponse response)
	{
		HttpEntity entity = response.getEntity();
		
		if (entity == null)
		{
			throw new IllegalArgumentException("HTTP entity may not be null");
		}
		
		if (entity.getContentLength() > Integer.MAX_VALUE)
		{
			throw new IllegalArgumentException("HTTP entity too large to be buffered in memory");
		}
			
		try
		{
			InputStream instream = entity.getContent();
			
			if (instream == null)
			{
				throw new IllegalArgumentException("Empty HTTP entity");
			}
			
			return instream;
		}
		catch ( IOException e )
		{
			throw new DbInvalidContentException("Error trying to retrieve the content", e);
		}
	}
	
	_MovieCasts _getMovieCasts(_MovieResult result)
	{
		String movieUrl = String.format(_DbConfig.MOVIE_CASTS_URL, result.id);
		HttpResponse response = _getResponse( movieUrl );
		String body = _getResponseBody(response);
		return processMovieCasts(body);	
	}
	
	private _MovieCasts processMovieCasts(String body)
	{
		return gson_.fromJson(body, _MovieCasts.class);
	}

	_MovieInfo _getMovieDetails(_MovieResult result)
	{
		String movieUrl = String.format(_DbConfig.MOVIE_INFO_URL, result.id);
		HttpResponse response = _getResponse( movieUrl );
		String body = _getResponseBody(response);
		return processMovieInfo(body);
	}
	
	private _MovieInfo processMovieInfo(String body)
	{
		return gson_.fromJson(body, _MovieInfo.class);
	}

	String _getResponseBody(HttpResponse response)
	{
		
		InputStream instream = _getInputStream(response);
		
		HttpEntity entity = response.getEntity();	
		
		String charset = null;
		if (entity.getContentType() != null)
		{
			HeaderElement values[] = entity.getContentType().getElements();
			if (values.length > 0)
			{
				NameValuePair param = values[0].getParameterByName("charset");
				if (param != null)
				{
					charset = param.getValue();
				}
			}
		}
		if (charset == null)
		{
			charset = HTTP.DEFAULT_CONTENT_CHARSET;
		}
	
		try
		{
			Reader reader = new InputStreamReader(instream, charset);
			StringBuilder buffer = new StringBuilder();
			
			try {
				char[] tmp = new char[1024];
				int l;
				while ((l = reader.read(tmp)) != -1)
				{
					buffer.append(tmp, 0, l);
				}
			} finally {
				reader.close();
			}
			return buffer.toString();
		}
		catch ( IOException e )
		{
			throw new DbInvalidContentException("Error trying to retrieve the content", e);
		}
	}
}
