package org.moviepedia.moviedb;

import java.io.File;
import java.io.InputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.http.HttpResponse;
import org.moviepedia.moviedb._MovieCasts._Cast;


class _MovieImpl implements Movie {

	private final _MovieDbImpl movieDb_;
	private final _MovieResult result_;
	private final String id_;
	private final Date releaseDate_;
	private final String posterUrl_;
	private final _MovieCasts casts_;
	private final Map<String, List<_Cast>> castMap_; 
	
	public _MovieImpl(_MovieDbImpl movieDb, _MovieResult result) {
		movieDb_ = movieDb;
		result_ = result;

		id_ = Integer.toString(result_.id);
		releaseDate_ = extractReleaseDate(result);
		posterUrl_ = extractPosterUrl();
		casts_ = movieDb_._getMovieCasts(result);
		
		Map<String, List<_Cast>> castMap = new HashMap<String, List<_Cast>>();
		if ( casts_ != null )
		{
			for ( _Cast cast : casts_.casts )
			{
				if ( cast.department != null )
				{
					addCastToMap(castMap, cast, cast.department);
				}
				if ( cast.character != null )
				{
					addCastToMap(castMap, cast, cast.character);
				}
			}
		}
		castMap_ = Collections.unmodifiableMap(castMap);
	}

	private void addCastToMap(Map<String, List<_Cast>> castMap, _Cast cast, String key) {
		List<_Cast> casts = castMap.get(key);
		if ( casts == null )
		{
			casts = new ArrayList<_Cast>();
			castMap.put(key, casts);
		}
		casts.add(cast);
	}

	private String extractPosterUrl() {
		_Image image = movieDb_.configResponse_.image;
		if ( result_.posterPath != null )
		{
			return image.baseUrl + image.posterSizes[0] + result_.posterPath;
		}
		else
		{
			return null;
		}
	}

	private Date extractReleaseDate(_MovieResult result) {
		Date releaseDate = null;
		if ( result.releaseDate != null && result.releaseDate.trim().length()>0 )
		{
			try {
				SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd", Locale.US);
				releaseDate = dateFormat.parse(result.releaseDate);
			} catch (ParseException e) {}
		}
		return releaseDate;
	}

	public String getId()
	{
		return id_;
	}
	
	public Date getReleaseDate() {
		return releaseDate_;
	}

	public String getTitle() {
		return result_.title;
	}

	public String getWriters()
	{
		String writers = "";
		castMap_.toString();
//		for ( _MovieCasts._Cast cast : castMap_.get("Directing") )
//		{
//			
//		}
		return writers;
	}
	
	public InputStream getPoster()
	{
		if ( posterUrl_ != null )
		{
			HttpResponse response = movieDb_._getResponse( posterUrl_ );
			return movieDb_._getInputStream( response );
		}
		return null;
	}
	
	public String getPosterUrl() {
//		if ( posterUrl_ != null )
//		{
//			try {
//				return new URI(posterUrl_);
//			} catch (URISyntaxException e) {
//				throw new DbException("Url not valid", e);
//			}
//		}
		return posterUrl_;
	}
}
